#include "DRMwraper.h"
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>

drmModeConnector *conn;
drmModeRes *res;
drmModePlaneRes *plane_res;
uint32_t conn_id;
uint32_t crtc_id;
uint32_t plane_id;

void initDRM(int fd, struct drmHandle *buf)
{
    // 获取crtc、encoder、connector属性信息
    res = drmModeGetResources(fd);
    crtc_id = res->crtcs[0];
    conn_id = res->connectors[0];

    // 获取plane_id
    int aa = drmSetClientCap(fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 1);

    plane_res = drmModeGetPlaneResources(fd);
    if(plane_res == NULL)
    {
        perror("drmModeGetPlaneResources() failed");
        exit(0);
    }

    plane_id = plane_res->planes[0];

    // 获取显示器尺寸
    conn = drmModeGetConnector(fd, conn_id);
    buf->width  = conn->modes[0].hdisplay;
    buf->height = conn->modes[0].vdisplay;
}


int createFB(int fd, struct drmHandle *drm)
{
    struct drm_mode_create_dumb create = {};
    struct drm_mode_map_dumb map = {};

    /* create a dumb-buffer, the pixel format is XRGB888 */
    create.width = drm->width;
    create.height = drm->height;
    create.bpp = 32;
    // 创建一块连续物理内存，大小等于显示屏的大小
    // 并获取行距、色深等信息
    drmIoctl(fd, DRM_IOCTL_MODE_CREATE_DUMB, &create);

    /* bind the dumb-buffer to an FB object */
    drm->pitch = create.pitch;
    drm->size = create.size;
    drm->handle = create.handle;
    drmModeAddFB(fd, drm->width, drm->height, 24/*depth*/, 32/*bpp*/, drm->pitch,
               drm->handle, &drm->fb_id);

    /* map the dumb-buffer to userspace */
    map.handle = create.handle;
    int k;
    if((k=drmIoctl(fd, DRM_IOCTL_MODE_MAP_DUMB, &map)) != 0)
    {
        printf("[%d]error: %s\n", __LINE__, strerror(errno));
        exit(0);
    }

    drm->vaddr = mmap(0, create.size, PROT_READ | PROT_WRITE,
            MAP_SHARED, fd, map.offset);
    if(drm->vaddr == MAP_FAILED)
    {
        perror("mmap() failed");
        exit(0);
    }
    else
        printf("映射 DRM 设备到用户内存空间成功\n");

    return 0;
}
